export declare class Html5Entities {
    decode(str: string): string;
    static decode(str: string): string;
    encode(str: string): string;
    static encode(str: string): string;
    encodeNonUTF(str: string): string;
    static encodeNonUTF(str: string): string;
    encodeNonASCII(str: string): string;
    static encodeNonASCII(str: string): string;
}
