/**
 * Tries to resolve the hostname. Returns true if succeeds.
 *
 * @param {String} host is the hostname from the URL.
 * @return {Boolean}
 */
export default function isResolvable(host: string): Promise<boolean>;
//# sourceMappingURL=isResolvable.d.ts.map